package it.dsestili.jhash.gui;

/*
JHash a simple hash code generator
Copyright (C) 2013 Davide Sestili

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import it.dsestili.jhash.core.DirectoryWorkerThread;
import it.dsestili.jhash.core.WorkerThread;
import java.awt.EventQueue;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Toolkit;

import javax.swing.border.BevelBorder;
import javax.swing.JButton;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

@SuppressWarnings("serial")
public class MainWindow extends JFrame 
{
	private WorkerThread workerThread = null;
	
	private JPanel contentPane;
	private InputPanel inputPanel;
	private StatusPanel statusPanel;
	private JButton btnStart;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel("javax.swing.plaf.nimbus.NimbusLookAndFeel");
		} catch (Throwable e) {
			e.printStackTrace();
		}
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					MainWindow frame = new MainWindow();
					centerScreen(frame);
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	public static void centerScreen(Component window)
	{
	    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
	    
	    int w = window.getSize().width;
	    int h = window.getSize().height;
	    int x = (dim.width - w) / 2;
	    int y = (dim.height - h) / 2;
	    
	    window.setLocation(x, y);	
	}

	private void closeApp()
	{
		if(workerThread != null)
		{
			workerThread.interruptOperation();
			try 
			{
				workerThread.join();
			} 
			catch (InterruptedException e) 
			{

			}
		}
		
		System.exit(0);
	}

	private void about()
	{
		AboutDialog dialog = new AboutDialog(this);
		centerScreen(dialog);
	}
	
	/**
	 * Create the frame.
	 */
	public MainWindow() {
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent arg0) {
				closeApp();
			}
		});
		setResizable(false);
		setTitle("JHash 0.3");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 841, 435);
		
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		JMenu mnFile = new JMenu("File");
		menuBar.add(mnFile);
		
		JMenuItem menuExit = new JMenuItem("Exit");
		menuExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				closeApp();
			}
		});
		mnFile.add(menuExit);
		
		JMenu mnNewMenu = new JMenu("   ?   ");
		menuBar.add(mnNewMenu);
		
		JMenuItem menuAbout = new JMenuItem("About");
		menuAbout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				about();
			}
		});
		mnNewMenu.add(menuAbout);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		GridBagLayout gbl_contentPane = new GridBagLayout();
		gbl_contentPane.columnWidths = new int[]{535, 0};
		gbl_contentPane.rowHeights = new int[]{129, 37, 0, 0};
		gbl_contentPane.columnWeights = new double[]{1.0, Double.MIN_VALUE};
		gbl_contentPane.rowWeights = new double[]{0.0, 0.0, 1.0, Double.MIN_VALUE};
		contentPane.setLayout(gbl_contentPane);
		
		inputPanel = new InputPanel();
		inputPanel.setBorder(new BevelBorder(BevelBorder.RAISED, null, null, null, null));
		GridBagConstraints gbc_inputPanel = new GridBagConstraints();
		gbc_inputPanel.fill = GridBagConstraints.HORIZONTAL;
		gbc_inputPanel.insets = new Insets(10, 10, 10, 10);
		gbc_inputPanel.gridx = 0;
		gbc_inputPanel.gridy = 0;
		contentPane.add(inputPanel, gbc_inputPanel);
		
		btnStart = new JButton("Generate Hash");
		btnStart.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				validateInput();
			}
		});
		GridBagConstraints gbc_btnStart = new GridBagConstraints();
		gbc_btnStart.fill = GridBagConstraints.HORIZONTAL;
		gbc_btnStart.insets = new Insets(10, 10, 10, 10);
		gbc_btnStart.gridx = 0;
		gbc_btnStart.gridy = 1;
		contentPane.add(btnStart, gbc_btnStart);
		
		statusPanel = new StatusPanel();
		statusPanel.setBorder(new BevelBorder(BevelBorder.LOWERED, null, null, null, null));
		GridBagConstraints gbc_statusPanel = new GridBagConstraints();
		gbc_statusPanel.insets = new Insets(10, 10, 10, 10);
		gbc_statusPanel.fill = GridBagConstraints.BOTH;
		gbc_statusPanel.gridx = 0;
		gbc_statusPanel.gridy = 2;
		contentPane.add(statusPanel, gbc_statusPanel);

		//pack();
	}

	private void validateInput()
	{
		if(inputPanel.isEnabled())
		{
			File file = new File(inputPanel.getFileName());

			if(!file.exists())
			{
				JOptionPane.showMessageDialog(this, "File \"" + file.getName() + "\" does not exist!", "Warning", JOptionPane.WARNING_MESSAGE);
				return;
			}

			if(!file.isFile())
			{
				workerThread = new DirectoryWorkerThread(this, file, inputPanel.getAlgorithm(), statusPanel, inputPanel, btnStart);
				workerThread.start();
			}
			else
			{
				workerThread = new WorkerThread(file, inputPanel.getAlgorithm(), statusPanel, inputPanel, btnStart);
				workerThread.start();
			}
		}
		else
		{
			workerThread.interruptOperation();
		}
	}
	
}
